#!/bin/sh -e
# Copyright @BABOLO (C)2002,2003
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: up.m4,v 1.1 2004/07/16 14:37:36 gd Exp $

genclass=mysql
suffix=
inclass=mysql
. ${JAILUP_CONF}
kind=`${dirname} "${0}"`
kind=`${basename} "${kind}"`
t=`${basename} "${0}"`
if  [ "${t}" != "${t#*-}" ]; then
    kind="${kind}/-${t#*-}"
fi

if  [ -z "${1}" ]; then
    ${cat} << EOF
# jailup ${inclass} dir jailname jailaddr
#     where jailname - hostname for jail (see jail(2) hostname)
#           jailaddr - IP addr for jail (see jail(2) ip_number)
#           dir relative to ${JAILUPDIR} and ${JAILRODIR}
# installed ports/databases/mysql40-server used
EOF
    exit
fi
umask 022

jdir="${1}"
jailsname="${2}"
jailsaddr="${3}"
main_dir="${JAILUPDIR}/${jdir}"
conf_dir="${JAILRODIR}/${jdir}"

${mkdir} "${main_dir}" "${conf_dir}"
${up_mdir} "${main_dir}" ${MTREE}

${up_shlib} "${main_dir}/usr/lib" << "EOF"
    c_r crypt m util wrap z
EOF

${up_libexe} "${main_dir}/usr/libexec" << EOF
    ld-elf.so.1
    mysqld
EOF

${up_data} "${main_dir}/usr/share/mysql" /usr/local/share/mysql/ << EOF
    /charsets/Index
    /charsets/cp1251.conf
    /charsets/koi8_ru.conf
    /charsets/latin1.conf
    /english/errmsg.sys
    /english/errmsg.txt
EOF

      #####################################
      ####                             ####
      #### Begin of etc fill           ####
      ####                             ####
${up_ponf} "${conf_dir}/my.cnf" << "EOF"
[mysqld]
socket          = /tmp/mysql.sock
skip-locking
key_buffer = 16K
max_allowed_packet = 1M
table_cache = 4
sort_buffer_size = 64K
net_buffer_length = 2K
thread_stack = 64K
tmpdir=/tmp
skip-networking
server-id       = 1
skip-bdb
skip-innodb
basedir = /usr
default-character-set = koi8_ru
user = mysql
EOF

${up_ponf} "${conf_dir}/group" << "EOF"
wheel:*:0:root
mysql:*:88:
EOF

${up_ponf} "${conf_dir}/master.passwd" << "EOF"
root:*:0:0::0:0:Charlie &:/:/nonexistent
mysql:*:88:88::0:0:Mysql server:/var/db/mysql:/nonexistent
EOF
      ####                             ####
      #### End of etc fill             ####
      ####                             ####
      #####################################

${up_mtree} "${main_dir}" ${MTREE}
${mtree} -Ueqxp "${conf_dir}" << "EOF"
/set type=file uname=root gname=wheel mode=0644 flags=none
.				type=dir mode=0755
    group
    master.passwd		mode=0600
    my.cnf
EOF

${pwd_mkdb} -d "${conf_dir}" -p "${conf_dir}/master.passwd"

${cat} << EOF >> "${genconfig}"
#${genclass}	${inclass}	${jdir}	${jailsname}	${jailsaddr}	0	0	0
EOF
      #####################################
      ####                             ####
      #### Begin of fstab mod          ####
      ####                             ####
${cat} << EOF >> "${etcdir}/fstab"
#${conf_dir}		${main_dir}/etc	null	ro,nodev,noexec	0 0
EOF
      ####                             ####
      #### End of fstab mod            ####
      ####                             ####
      #####################################

# Initialize variables
c_d=""
c_h="" i_h=""
c_u="" i_u=""
c_f="" i_f=""
c_t="" c_c=""

mysql_db="${main_dir}/var/db/mysql/mysql"

# Check for old tables
if [ ! -f ${mysql_db}/db.frm ]
then
  echo "Preparing db table"

  # mysqld --bootstrap wants one command/line
  c_d="${c_d} CREATE TABLE db ("
  c_d="${c_d}   Host char(60) binary DEFAULT '' NOT NULL,"
  c_d="${c_d}   Db char(64) binary DEFAULT '' NOT NULL,"
  c_d="${c_d}   User char(16) binary DEFAULT '' NOT NULL,"
  c_d="${c_d}   Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d}   Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_d="${c_d} PRIMARY KEY Host (Host,Db,User),"
  c_d="${c_d} KEY User (User)"
  c_d="${c_d} )"
  c_d="${c_d} comment='Database privileges';"
  
fi

if [ ! -f ${mysql_db}/host.frm ]
then
  echo "Preparing host table"

  c_h="${c_h} CREATE TABLE host ("
  c_h="${c_h}  Host char(60) binary DEFAULT '' NOT NULL,"
  c_h="${c_h}  Db char(64) binary DEFAULT '' NOT NULL,"
  c_h="${c_h}  Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_h="${c_h}  PRIMARY KEY Host (Host,Db)"
  c_h="${c_h} )"
  c_h="${c_h} comment='Host privileges;  Merged with database privileges';"
fi

if [ ! -f ${mysql_db}/user.frm ]
then
  echo "Preparing user table"

  c_u="${c_u} CREATE TABLE user ("
  c_u="${c_u}   Host char(60) binary DEFAULT '' NOT NULL,"
  c_u="${c_u}   User char(16) binary DEFAULT '' NOT NULL,"
  c_u="${c_u}   Password char(16) binary DEFAULT '' NOT NULL,"
  c_u="${c_u}   Select_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Insert_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Update_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Delete_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Create_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Drop_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Reload_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Shutdown_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Process_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   File_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Grant_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   References_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Index_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Alter_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Show_db_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Super_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Execute_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Repl_slave_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   Repl_client_priv enum('N','Y') DEFAULT 'N' NOT NULL,"
  c_u="${c_u}   ssl_type enum('','ANY','X509', 'SPECIFIED') DEFAULT '' NOT NULL,"
  c_u="${c_u}   ssl_cipher BLOB NOT NULL,"
  c_u="${c_u}   x509_issuer BLOB NOT NULL,"
  c_u="${c_u}   x509_subject BLOB NOT NULL,"
  c_u="${c_u}   max_questions int(11) unsigned DEFAULT 0  NOT NULL,"
  c_u="${c_u}   max_updates int(11) unsigned DEFAULT 0  NOT NULL,"
  c_u="${c_u}   max_connections int(11) unsigned DEFAULT 0  NOT NULL,"
  c_u="${c_u}   PRIMARY KEY Host (Host,User)"
  c_u="${c_u} )"
  c_u="${c_u} comment='Users and global privileges';"

  i_u="INSERT INTO user VALUES ('localhost','root','','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','','','','',0,0,0);
  
  REPLACE INTO user VALUES ('localhost','root','','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','Y','','','','',0,0,0);
  
  INSERT INTO user (host,user) values ('localhost','');"
fi

if [ ! -f ${mysql_db}/func.frm ]
then
  echo "Preparing func table"

  c_f="${c_f} CREATE TABLE func ("
  c_f="${c_f}   name char(64) binary DEFAULT '' NOT NULL,"
  c_f="${c_f}   ret tinyint(1) DEFAULT '0' NOT NULL,"
  c_f="${c_f}   dl char(128) DEFAULT '' NOT NULL,"
  c_f="${c_f}   type enum ('function','aggregate') NOT NULL,"
  c_f="${c_f}   PRIMARY KEY (name)"
  c_f="${c_f} )"
  c_f="${c_f}   comment='User defined functions';"
fi

if [ ! -f ${mysql_db}/tables_priv.frm ]
then
  echo "Preparing tables_priv table"

  c_t="${c_t} CREATE TABLE tables_priv ("
  c_t="${c_t}   Host char(60) binary DEFAULT '' NOT NULL,"
  c_t="${c_t}   Db char(64) binary DEFAULT '' NOT NULL,"
  c_t="${c_t}   User char(16) binary DEFAULT '' NOT NULL,"
  c_t="${c_t}   Table_name char(60) binary DEFAULT '' NOT NULL,"
  c_t="${c_t}   Grantor char(77) DEFAULT '' NOT NULL,"
  c_t="${c_t}   Timestamp timestamp(14),"
  c_t="${c_t}   Table_priv set('Select','Insert','Update','Delete','Create','Drop','Grant','References','Index','Alter') DEFAULT '' NOT NULL,"
  c_t="${c_t}   Column_priv set('Select','Insert','Update','References') DEFAULT '' NOT NULL,"
  c_t="${c_t}   PRIMARY KEY (Host,Db,User,Table_name),"
  c_t="${c_t}   KEY Grantor (Grantor)"
  c_t="${c_t} )"
  c_t="${c_t}   comment='Table privileges';"
fi

if [ ! -f ${mysql_db}/columns_priv.frm ]
then
  echo "Preparing columns_priv table"

  c_c="${c_c} CREATE TABLE columns_priv ("
  c_c="${c_c}   Host char(60) binary DEFAULT '' NOT NULL,"
  c_c="${c_c}   Db char(64) binary DEFAULT '' NOT NULL,"
  c_c="${c_c}   User char(16) binary DEFAULT '' NOT NULL,"
  c_c="${c_c}   Table_name char(64) binary DEFAULT '' NOT NULL,"
  c_c="${c_c}   Column_name char(64) binary DEFAULT '' NOT NULL,"
  c_c="${c_c}   Timestamp timestamp(14),"
  c_c="${c_c}   Column_priv set('Select','Insert','Update','References') DEFAULT '' NOT NULL,"
  c_c="${c_c}   PRIMARY KEY (Host,Db,User,Table_name,Column_name)"
  c_c="${c_c} )"
  c_c="${c_c}   comment='Column privileges';"
fi

echo "Installing all prepared tables"
${jail} ${main_dir} ${jailsname} ${jailsaddr} /usr/libexec/mysqld -b /usr --bootstrap --skip-grant-tables --datadir=/var/db/mysql --skip-innodb --skip-bdb --tmpdir=/tmp << EOF
use mysql;
${c_d}
${c_h}
${i_h}
${c_u}
${i_u}
${c_f}
${i_f}
${c_t}
${c_c}
EOF
